// This file was generated by qtwaylandscanner
// source file is fake-input.xml

#include "qwayland-fake-input.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    org_kde_kwin_fake_input::org_kde_kwin_fake_input(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    org_kde_kwin_fake_input::org_kde_kwin_fake_input(struct ::org_kde_kwin_fake_input *obj)
        : m_org_kde_kwin_fake_input(obj)
    {
    }

    org_kde_kwin_fake_input::org_kde_kwin_fake_input()
        : m_org_kde_kwin_fake_input(nullptr)
    {
    }

    org_kde_kwin_fake_input::~org_kde_kwin_fake_input()
    {
    }

    void org_kde_kwin_fake_input::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_org_kde_kwin_fake_input = static_cast<struct ::org_kde_kwin_fake_input *>(wlRegistryBind(registry, id, &org_kde_kwin_fake_input_interface, version));
    }

    void org_kde_kwin_fake_input::init(struct ::org_kde_kwin_fake_input *obj)
    {
        m_org_kde_kwin_fake_input = obj;
    }

    org_kde_kwin_fake_input *org_kde_kwin_fake_input::fromObject(struct ::org_kde_kwin_fake_input *object)
    {
        return static_cast<org_kde_kwin_fake_input *>(org_kde_kwin_fake_input_get_user_data(object));
    }

    bool org_kde_kwin_fake_input::isInitialized() const
    {
        return m_org_kde_kwin_fake_input != nullptr;
    }

    uint32_t org_kde_kwin_fake_input::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_org_kde_kwin_fake_input));
    }

    const struct wl_interface *org_kde_kwin_fake_input::interface()
    {
        return &::org_kde_kwin_fake_input_interface;
    }

    void org_kde_kwin_fake_input::authenticate(const QString &application, const QString &reason)
    {
        ::org_kde_kwin_fake_input_authenticate(
            m_org_kde_kwin_fake_input,
            application.toUtf8().constData(),
            reason.toUtf8().constData());
    }

    void org_kde_kwin_fake_input::pointer_motion(wl_fixed_t delta_x, wl_fixed_t delta_y)
    {
        ::org_kde_kwin_fake_input_pointer_motion(
            m_org_kde_kwin_fake_input,
            delta_x,
            delta_y);
    }

    void org_kde_kwin_fake_input::button(uint32_t button, uint32_t state)
    {
        ::org_kde_kwin_fake_input_button(
            m_org_kde_kwin_fake_input,
            button,
            state);
    }

    void org_kde_kwin_fake_input::axis(uint32_t axis, wl_fixed_t value)
    {
        ::org_kde_kwin_fake_input_axis(
            m_org_kde_kwin_fake_input,
            axis,
            value);
    }

    void org_kde_kwin_fake_input::touch_down(uint32_t id, wl_fixed_t x, wl_fixed_t y)
    {
        ::org_kde_kwin_fake_input_touch_down(
            m_org_kde_kwin_fake_input,
            id,
            x,
            y);
    }

    void org_kde_kwin_fake_input::touch_motion(uint32_t id, wl_fixed_t x, wl_fixed_t y)
    {
        ::org_kde_kwin_fake_input_touch_motion(
            m_org_kde_kwin_fake_input,
            id,
            x,
            y);
    }

    void org_kde_kwin_fake_input::touch_up(uint32_t id)
    {
        ::org_kde_kwin_fake_input_touch_up(
            m_org_kde_kwin_fake_input,
            id);
    }

    void org_kde_kwin_fake_input::touch_cancel()
    {
        ::org_kde_kwin_fake_input_touch_cancel(
            m_org_kde_kwin_fake_input);
    }

    void org_kde_kwin_fake_input::touch_frame()
    {
        ::org_kde_kwin_fake_input_touch_frame(
            m_org_kde_kwin_fake_input);
    }

    void org_kde_kwin_fake_input::pointer_motion_absolute(wl_fixed_t x, wl_fixed_t y)
    {
        ::org_kde_kwin_fake_input_pointer_motion_absolute(
            m_org_kde_kwin_fake_input,
            x,
            y);
    }

    void org_kde_kwin_fake_input::keyboard_key(uint32_t button, uint32_t state)
    {
        ::org_kde_kwin_fake_input_keyboard_key(
            m_org_kde_kwin_fake_input,
            button,
            state);
    }

    void org_kde_kwin_fake_input::destroy()
    {
        ::org_kde_kwin_fake_input_destroy(
            m_org_kde_kwin_fake_input);
        m_org_kde_kwin_fake_input = nullptr;
    }

    void org_kde_kwin_fake_input::keyboard_keysym(uint32_t keysym, uint32_t state)
    {
        ::org_kde_kwin_fake_input_keyboard_keysym(
            m_org_kde_kwin_fake_input,
            keysym,
            state);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
