/****************************************************************************
** Meta object code from reading C++ file 'kcmkrdpserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../krdp-6.7.0/src/kcm/kcmkrdpserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcmkrdpserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16KRDPServerConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto KRDPServerConfig::qt_create_metaobjectdata<qt_meta_tag_ZN16KRDPServerConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KRDPServerConfig",
        "krdpServerSettingsChanged",
        "",
        "generateCertificateSucceeded",
        "generateCertificateFailed",
        "passwordLoaded",
        "user",
        "password",
        "keychainError",
        "errorText",
        "serverStatusChanged",
        "errorMessageChanged",
        "isServerRunningChanged",
        "save",
        "defaults",
        "servicePropertiesChanged",
        "toLocalFile",
        "QUrl",
        "url",
        "modifyUser",
        "oldUsername",
        "newUsername",
        "newPassword",
        "addUser",
        "username",
        "deleteUser",
        "userExists",
        "readPasswordFromWallet",
        "isH264Supported",
        "listenAddressList",
        "toggleAutoconnect",
        "enabled",
        "toggleServer",
        "restartServer",
        "generateCertificate",
        "copyAddressToClipboard",
        "address",
        "settings",
        "KRDPServerSettings*",
        "updateServerStatus",
        "isServerRunning",
        "serverStatus",
        "SystemdService::Status",
        "errorMessage",
        "hostName",
        "managementAvailable",
        "users",
        "QAbstractItemModel*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'krdpServerSettingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'generateCertificateSucceeded'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'generateCertificateFailed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordLoaded'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'keychainError'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'serverStatusChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorMessageChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isServerRunningChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'defaults'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'servicePropertiesChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'toLocalFile'
        QtMocHelpers::MethodData<QString(const QUrl &)>(16, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'modifyUser'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::QString, 21 }, { QMetaType::QString, 22 },
        }}),
        // Method 'addUser'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { QMetaType::QString, 7 },
        }}),
        // Method 'deleteUser'
        QtMocHelpers::MethodData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Method 'userExists'
        QtMocHelpers::MethodData<bool(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 24 },
        }}),
        // Method 'readPasswordFromWallet'
        QtMocHelpers::MethodData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'isH264Supported'
        QtMocHelpers::MethodData<bool()>(28, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'listenAddressList'
        QtMocHelpers::MethodData<QStringList()>(29, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'toggleAutoconnect'
        QtMocHelpers::MethodData<void(const bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Method 'toggleServer'
        QtMocHelpers::MethodData<void(const bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Method 'restartServer'
        QtMocHelpers::MethodData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'generateCertificate'
        QtMocHelpers::MethodData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'copyAddressToClipboard'
        QtMocHelpers::MethodData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Method 'settings'
        QtMocHelpers::MethodData<KRDPServerSettings *() const>(37, 2, QMC::AccessPublic, 0x80000000 | 38),
        // Method 'updateServerStatus'
        QtMocHelpers::MethodData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isServerRunning'
        QtMocHelpers::MethodData<bool() const>(40, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'serverStatus'
        QtMocHelpers::PropertyData<SystemdService::Status>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'hostName'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'managementAvailable'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'users'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KRDPServerConfig, qt_meta_tag_ZN16KRDPServerConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN16KRDPServerConfigE[] = {
    QMetaObject::SuperData::link<SystemdService::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KRDPServerConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<KQuickManagedConfigModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KRDPServerConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KRDPServerConfigE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN16KRDPServerConfigE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16KRDPServerConfigE_t>.metaTypes,
    nullptr
} };

void KRDPServerConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KRDPServerConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->krdpServerSettingsChanged(); break;
        case 1: _t->generateCertificateSucceeded(); break;
        case 2: _t->generateCertificateFailed(); break;
        case 3: _t->passwordLoaded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->keychainError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->serverStatusChanged(); break;
        case 6: _t->errorMessageChanged(); break;
        case 7: _t->isServerRunningChanged(); break;
        case 8: _t->save(); break;
        case 9: _t->defaults(); break;
        case 10: _t->servicePropertiesChanged(); break;
        case 11: { QString _r = _t->toLocalFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->modifyUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 13: _t->addUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->deleteUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: { bool _r = _t->userExists((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->readPasswordFromWallet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: { bool _r = _t->isH264Supported();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { QStringList _r = _t->listenAddressList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->toggleAutoconnect((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->toggleServer((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->restartServer(); break;
        case 22: _t->generateCertificate(); break;
        case 23: _t->copyAddressToClipboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: { KRDPServerSettings* _r = _t->settings();
            if (_a[0]) *reinterpret_cast<KRDPServerSettings**>(_a[0]) = std::move(_r); }  break;
        case 25: _t->updateServerStatus(); break;
        case 26: { bool _r = _t->isServerRunning();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::krdpServerSettingsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::generateCertificateSucceeded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::generateCertificateFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)(const QString & , const QString & )>(_a, &KRDPServerConfig::passwordLoaded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)(const QString & )>(_a, &KRDPServerConfig::keychainError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::serverStatusChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::errorMessageChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KRDPServerConfig::*)()>(_a, &KRDPServerConfig::isServerRunningChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<SystemdService::Status*>(_v) = _t->serverStatus(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->errorMessage(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->hostName(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->managementAvailable(); break;
        case 4: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->usersModel(); break;
        default: break;
        }
    }
}

const QMetaObject *KRDPServerConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KRDPServerConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16KRDPServerConfigE_t>.strings))
        return static_cast<void*>(this);
    return KQuickManagedConfigModule::qt_metacast(_clname);
}

int KRDPServerConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KQuickManagedConfigModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KRDPServerConfig::krdpServerSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KRDPServerConfig::generateCertificateSucceeded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KRDPServerConfig::generateCertificateFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KRDPServerConfig::passwordLoaded(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void KRDPServerConfig::keychainError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KRDPServerConfig::serverStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KRDPServerConfig::errorMessageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KRDPServerConfig::isServerRunningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
namespace {
struct qt_meta_tag_ZN14SystemdServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto SystemdService::qt_create_metaobjectdata<qt_meta_tag_ZN14SystemdServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SystemdService",
        "Status",
        "Unknown",
        "Running",
        "Stopped",
        "Failed"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<Status>(1, 1, QMC::EnumFlags{}).add({
            {    2, Status::Unknown },
            {    3, Status::Running },
            {    4, Status::Stopped },
            {    5, Status::Failed },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN14SystemdServiceE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN14SystemdServiceE =
    SystemdService::qt_create_metaobjectdata<qt_meta_tag_ZN14SystemdServiceE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN14SystemdServiceE =
    qt_staticMetaObjectContent_ZN14SystemdServiceE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN14SystemdServiceE =
    qt_staticMetaObjectContent_ZN14SystemdServiceE.relocatingData;

Q_CONSTINIT const QMetaObject SystemdService::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN14SystemdServiceE.stringdata,
    qt_staticMetaObjectStaticContent_ZN14SystemdServiceE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN14SystemdServiceE.metaTypes,
    nullptr
} };

QT_WARNING_POP
